@@ -1,5 +1,5 @@
 /*
- * JointPrior.h
+ * PriorFunction.h
  *
  *  Created on: Nov 11, 2013
  *      Author: msuchard
@@ -46,7 +46,9 @@ public:
 //  	virtual JointPrior* clone() const = 0; // pure virtual
 
     void addVarianceParameter(const VariancePtr& ptr) {
-        variance.push_back(ptr); // TODO Check for uniqueness
+        if (std::find(variance.begin(), variance.end(), ptr) == variance.end()) {
+            variance.push_back(ptr);
+        }
     }
 
 	void addVarianceParameters(const std::vector<VariancePtr>& ptrs) {
@@ -57,15 +59,15 @@ public:
 
 	void setVariance(int index, double x) {
 		// TODO Check bounds
-		*variance[index] = x;
+		variance[index].set(x);
 	}
 
 	std::vector<double> getVariance() const {
 	    std::vector<double> tmp;
 	    for (auto v : variance) {
-	        tmp.push_back(*v);
+	        tmp.push_back(v.get());
 	    }
-		return std::move(tmp);
+		return tmp;
 	}
 
 protected:
@@ -89,6 +91,7 @@ public:
 		// TODO assert(index < listPriors.size());
 		listPriors[index] = newPrior;
 		uniquePriors.push_back(newPrior);
+		addVarianceParameters(newPrior->getVarianceParameters());
 	}
 
 	const std::string getDescription() const {
@@ -532,8 +535,8 @@ public:
 // 		double t1 = 1/hierarchyPriors[0]->getVariance(0); // this is the hyperparameter that is used in the original code
 // 		double t2 = 1/hierarchyPriors[1]->getVariance(0);
 
-		double t1 = 1.0 / *variance[0];
-		double t2 = 1.0 / *variance[1];
+		double t1 = 1.0 / variance[0].get();
+		double t2 = 1.0 / variance[1].get();
 
 		int parent = getParentMap.at(index);
 		const std::vector<int>& siblings = getChildMap.at(parent);
